# Copyright 2015 Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Jinja templating for Fastfood."""

import os
import re

import jinja2

# create a jinja env, overriding delimiters
JINJA_ENV = jinja2.Environment(variable_start_string='|{',
                               variable_end_string='}|',
                               trim_blocks=True)
NODE_ATTR_RE = '^node((\[\'([\w_-]+)\'\])+)'
CHEF_CONST_RE = '^node\.([\w_-]+)'


def qstring(option):
    if (re.match(NODE_ATTR_RE, option) is None and
            re.match(CHEF_CONST_RE, option) is None):
        return "'%s'" % option
    else:
        return option


JINJA_ENV.globals['qstring'] = qstring


def render_templates(*files, **template_map):
    """Render jinja templates according to template_map.

    Return a list of [(path, result), (...)]
    """
    return list(render_templates_generator(*files, **template_map))


def render_templates_generator(*files, **template_map):
    """Render jinja templates according to template_map.

    Yields (path, result)
    """
    for path in files:
        if not os.path.isfile(path):
            raise ValueError("Template file %s not found"
                             % os.path.relpath(path))
        else:
            try:
                template = JINJA_ENV.from_string(open(path).read())
            except jinja2.TemplateSyntaxError as err:
                msg = ("Error rendering jinja2 template for file %s "
                       "on line %s. Error: %s"
                       % (path, err.lineno, err.message))
                raise type(err)(
                    msg, err.lineno, filename=os.path.basename(path))

            result = template.render(**template_map)
            yield path, result



# Copyright 2015 Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Fastfood Template Pack manager."""

import json
import os

from fastfood import stencil as stencil_module
from fastfood import utils


class TemplatePack(object):

    def __init__(self, path):
        """Initialize, asserting templatepack path and manifest."""
        self._manifest = None
        self._stencil_sets = {}
        self.path = utils.normalize_path(path)
        if not os.path.isdir(self.path):
            raise ValueError("Templatepack dir %s does not exist."
                             % self.path)
        self.manifest_path = os.path.join(self.path, 'manifest.json')
        if not os.path.isfile(self.manifest_path):
            raise ValueError("Templatepack needs manifest file, %s"
                             % os.path.relpath(self.manifest_path))
        self._validate('api', cls=int)
        self._validate('base', cls=dict)
        self._validate('stencil_sets', cls=dict)
        # for caching Stencil instances

    def _validate(self, key, cls=None):
        if key not in self.manifest:
            raise ValueError("Manifest %s requires '%s'."
                             % (self.manifest_path, key))
        if cls:
            if not isinstance(self.manifest[key], cls):
                raise TypeError("Manifest value '%s' should be %s, not %s"
                                % (key, cls, type(self.manifest[key])))

    @property
    def manifest(self):
        if not self._manifest:
            with open(self.manifest_path) as man:
                self._manifest = json.load(man)
        return self._manifest

    @property
    def stencil_sets(self):
        if not self._stencil_sets:
            self._stencil_sets = self.manifest['stencil_sets']
        return self._stencil_sets

    def __getattr__(self, stencilset_name):
        """Shortcut to self.load_stencil_set()."""
        return self.load_stencil_set(stencilset_name)

    def load_stencil_set(self, stencilset_name):
        """Return the Stencil Set from this template pack."""
        if stencilset_name not in self._stencil_sets:
            if stencilset_name not in self.manifest['stencil_sets'].keys():
                raise AttributeError("Stencil set '%s' not listed in %s under "
                                     "stencil_sets."
                                     % (stencilset_name, self.manifest_path))
            stencil_path = os.path.join(
                self.path, 'stencils', stencilset_name)
            try:
                self._stencil_sets[stencilset_name] = (
                    stencil_module.StencilSet(stencil_path))
            except ValueError as err:
                raise AttributeError(err.message)
        return self._stencil_sets[stencilset_name]



"""Functional tests for command line use."""

import errno
import os
import random
import shutil
import string
import tempfile
import unittest

from fastfood import pack
from fastfood import shell

TEST_TEMPLATEPACK = os.path.abspath(
    os.path.join(os.path.dirname(os.path.realpath(__file__)),
                 os.path.pardir, 'test_templatepack'))
assert os.path.isdir(TEST_TEMPLATEPACK), "Test templatepack not found."


class MockArgs(object):

    def __init__(self, **args):
        self._args = args
        for key, value in args.iteritems():
            setattr(self, key, value)

    def __repr__(self):
        return repr(self._args)


class TestFastfoodCommands(unittest.TestCase):

    def setUp(self):

        self.templatepack_path = TEST_TEMPLATEPACK
        self.pack = pack.TemplatePack(self.templatepack_path)
        # make a separate ref in case something modifies the instance attr
        self.cookbooks_path = self._tmpcbd = tempfile.mkdtemp(
            prefix='%s.' % __file__, suffix='.cookbooks_dir')
        cli_args = {
            'template_pack': self.templatepack_path,
            'cookbooks': self.cookbooks_path,
            'cookbook': None,
            'cookbook_name': None,
            'force': None,
            'options': None,  # stencil options
            'config_file': None,  # for `fastfood build`
        }
        self.args = MockArgs(**cli_args)

    def tearDown(self):
        try:
            shutil.rmtree(self._tmpcbd)
        except OSError as exc:
            if exc.errno != errno.ENOENT:
                raise


class TestFastfoodNewCommand(TestFastfoodCommands):

    def test_fastfood_new(self):

        self.args.cookbook_name = ''.join(
            [random.choice(string.ascii_letters) for _ in xrange(10)])
        written, _ = shell._fastfood_new(self.args)
        # assert that files in pack manifest match the files
        # created by `fastfood new`
        base_path = '%s/' % os.path.join(
            self.cookbooks_path, self.args.cookbook_name)
        actual_files = [k[len(base_path):] for k in written
                        if k.startswith(base_path)]
        base_manifest_files = self.pack.manifest['base']['files']
        self.assertItemsEqual(actual_files, base_manifest_files)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# Copyright 2015 Rackspace US, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Fastfood packaging and installation."""

import ast
import re
from setuptools import setup, find_packages


DEPENDENCIES = [
    'jinja2==2.7.3',
]
TESTS_REQUIRE = [
]


def package_meta():
    """Read __init__.py for global package metadata.

    Do this without importing the package.
    """
    _version_re = re.compile(r'__version__\s+=\s+(.*)')
    _url_re = re.compile(r'__url__\s+=\s+(.*)')
    _license_re = re.compile(r'__license__\s+=\s+(.*)')

    with open('fastfood/__init__.py', 'rb') as ffinit:
        initcontent = ffinit.read()
        version = str(ast.literal_eval(_version_re.search(
            initcontent.decode('utf-8')).group(1)))
        url = str(ast.literal_eval(_url_re.search(
            initcontent.decode('utf-8')).group(1)))
        licencia = str(ast.literal_eval(_license_re.search(
            initcontent.decode('utf-8')).group(1)))
    return {
        'version': version,
        'license': licencia,
        'url': url,
    }

_ff_meta = package_meta()


setup(
    name='fastfood',
    description='Chef Cookbook Wizardry',
    keywords='chef cookbook templates',
    version=_ff_meta['version'],
    tests_require=TESTS_REQUIRE,
    test_suite='tests',
    install_requires=DEPENDENCIES,
    packages=find_packages(exclude=['tests']),
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
    ],
    license=_ff_meta['license'],
    author='Rackers',
    maintainer_email='samuel.stavinoha@rackspace.com',
    url=_ff_meta['url'],
    entry_points={
        'console_scripts': [
            'fastfood=fastfood.shell:main'
        ]
    },
)
